#! /usr/bin/perl -w

my $PDIR = '..' ;
my $PSD = 'psd' ;
my $model = '101' ;

my @vg = (0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.7, 0.9, 1.0, 5.0) ;
my @vpi = (1.1, 1.2, 1.3, 1.4, 1.5, 10, 10000) ;
my $eps = 0.0001 ;

$table = 'table3.htm' ;
$title = 'Table 3  Monetary policy rules without discretion';
$fnote = 'Note:' ;

my $statfile = 'grid.dat' ;


#
# solve model and do simulation
#
my $msdfile = 'msd.dat' ;

copy_file("$PDIR/simu_hp.tsp","simu_hp.tsp") ;
copy_file("$PDIR/HPfilter.tsp","HPfilter.tsp") ;
write_file($statfile,"") ; # initialize

my $case = 0 ;
foreach my $vg (@vg) {
    foreach my $vpi (@vpi) {
	$_ = get_params($case,$vpi,$vg) ;
	write_file('CaseNo.def', $_) ;
	write_file($msdfile,"NA\n") ;
	system("$PSD $PDIR/steady.in") ;
	system("$PSD $PDIR/beliefs.in") ;
	system("$PSD $PDIR/RBE$model.in") ;

	++$case ;
	write_file(">$statfile","CaseNo = $case\n") ;
	copy_file($msdfile,">$statfile") ;
	write_file(">$statfile","\n") ;
    }
}

#
# make a html table
#
$NOB = 0 ;
$LOGY = $NOB + 1 ;
$LOGI = $LOGY + 1 ;
$LOGC = $LOGI + 1 ;
$LOGL = $LOGC + 1 ;
$LOGW = $LOGL + 1 ;
$PI   = $LOGW + 1 ;
$Err1 = $PI + 1 ;
$Err2 = $Err1 + 1 ;
$Err3 = $Err2 + 1 ;
$cPI  = $Err3 + 1 ;

$_ = read_file($statfile) ;
@records = split(/\s*CaseNo = [0-9]+\s*/) ;
$jmax = @vg ;
$imax = @vpi ;

open(FILE,">$table") || die("Cannot write to $table\n") ;
select FILE ;
write_table_htm() ;
select STDOUT ;
close(FILE) ;

exit ;

#
# get_params($caseno)
#
sub get_params {
    my ($case,$vpi,$vg) = @_ ;
    my $num = $case + 1 ;
    my $startREE = 1 ;
    #$startREE = 1 unless ($case) ;
    return qq~
const CaseNo = $num ,
  vg_0 = $vg, vpi_0 = $vpi, eps_star = $eps,
  MONEY_GROWTH = 0, start_with_REE = $startREE ;
    ~ ;
}

#
# read_file($src)
#
sub read_file {
    my ($src) = @_ ;
    my $delim = $/;
    $/ = undef ;
    open(FILE,"$src") || die("Cannot write to file $src\n") ;
    $_ = <FILE> ;
    close(FILE) ;
    $/ = $delim ;
    return $_ ;
}

#
# write_file($filename,$string)
#
sub write_file {
    my ($fn,$tx) = @_ ;
    open(FILE,">$fn") || die("Cannot write to file $fn\n") ;
    print FILE $tx ;
    close(FILE) ;
}

#
# copy_file($src,$dst) 
#
sub copy_file {
    my ($src,$dst) = @_ ;
    $_ = read_file($src) ;
    write_file($dst,$_) ;
}


#
# Create HTML table
#
sub write_table_htm {
    write_tabhead_htm() ;
    my ($i, $j, $idx) ;
    for($i = 0; $i < $imax; $i++) {
	write_rowhead_htm($i) ;
	for($j = 0; $j < $jmax; $j++) {
	    $idx = 1 + $i + $j*$imax ;
	    my $rec = '';
	    $rec = $records[$idx] if ($records[$idx]) ;
	    write_rec_htm($rec) ;	
	}
	print "</tr>\n";
    }
    write_tabtail_htm() ;
}
sub write_tabhead_htm {
    print "<html><body>\n" ;
    print "<h3>$title</h3>\n" ;
    print "<table border=1><tr align=center valign=middle><td>&nbsp;<td>&nbsp;";
    my $j ;
    for($j = 0; $j < $jmax; $j++) {
	print "<td>$vg[$j]";
    }
    print "</tr>\n" ;
}
sub write_tabtail_htm {
    print "</table>\n" ;
    print "<p>$fnote\n" ;
    print "</body></html>\n" ;
}
sub write_rowhead_htm {
    my ($i) = @_ ;
    print "<tr align=center valign=middle><td>$vpi[$i]\n" ;
    print "<td>Y<br>I<br>C<br><font face=\'Symbol\'>p</font><br>"
          ."cov(<font face=\'Symbol\'>p</font>,Y)\n";
}
sub write_rec_htm {
    my $string = $_[0] ;
    $string =~ s/^\s*//og ;
    $string =~ s/\s*$//og ;
    my @numarray = split(/\s+/, $string) ;

    if(defined($numarray[$LOGY])) {
	printf 
	    "<td>%1.2f<br>%1.2f<br>%1.2f<br>%1.2f<br>%1.2f\n",
	    $numarray[$LOGY] / (100 * $eps),
	    $numarray[$LOGI] / (100 * $eps),
	    $numarray[$LOGC] / (100 * $eps),
	    $numarray[$PI] / (100 * $eps) ,
	    $numarray[$cPI] ;
    } else {
	print "<td>unstable\n" ;
    }
}
